@@ -16,7 +16,7 @@ EXIT_PYTHON_TOO_OLD = 4
 
 # ---------- current working directory ---------
 
-def assert_git_repo():
+def assert_git_repo() -> None:
 	for arg in sys.argv[1:]:
 		if arg.startswith('--git-dir'):
 			return
@@ -54,47 +54,47 @@ def assert_git_repo():
 
 # ---------- urwid version ---------
 
-def check_urwid_version():
+def check_urwid_version() -> None:
 	if not is_urwid_new_enough_for_ellipsis(fallback=True):
 		from . import urwid_text_layout
 		urwid_text_layout.LogTextLayout.ALLOWED_WRAP_VALUES = tuple(wrap for wrap in urwid_text_layout.LogTextLayout.ALLOWED_WRAP_VALUES if wrap != urwid_text_layout.LogTextLayout.WRAP_ELLIPSIS)
 
-def is_urwid_new_enough(version, fallback=True, assumption=None):
+def is_urwid_new_enough(min_version: str, *, fallback: bool = True, assumption: 'str|None' = None) -> bool:
 	try:
 		from packaging import version
 		import urwid
-		return version.parse(urwid.__version__) >= version.parse("2.1.0")
+		return version.parse(urwid.__version__) >= version.parse(min_version)
 	except ImportError:
 		if assumption:
 			warning(_("Failed to check urwid version because packaging is not installed. %s You can install packaging using `pip3 install --user packaging`." % assumption.strip()))
 		return fallback
 
-def is_urwid_new_enough_for_ellipsis(**kw):
-	return is_urwid_new_enough('2.1.0', **kw)
+def is_urwid_new_enough_for_ellipsis(*, fallback: bool = True, assumption: 'str|None' = None) -> bool:
+	return is_urwid_new_enough('2.1.0', fallback=fallback, assumption=assumption)
 
-def is_urwid_new_enough_for_explicit_spaces(**kw):
-	return is_urwid_new_enough('2.1.0', **kw)
+def is_urwid_new_enough_for_explicit_spaces(*, fallback: bool = True, assumption: 'str|None' = None) -> bool:
+	return is_urwid_new_enough('2.1.0', fallback=fallback, assumption=assumption)
 
 
 # ---------- python version ---------
 
-def assert_python_new_enough():
+def assert_python_new_enough() -> None:
 	if sys.version_info <= (3, 5, 0):
 		error(EXIT_PYTHON_TOO_OLD, _("This program requires at least Python 3.5.\nYou are using Python {version}.").format(version=sys.version.replace("\n", "")))
 
 
 # ---------- main ---------
 
-def warning(message):
+def warning(message: str) -> None:
 	sys.stderr.write(message)
 	sys.stderr.write("\n")
 
-def error(error_number, error_message):
+def error(error_number: int, error_message: str) -> None:
 	sys.stderr.write(error_message)
 	sys.stderr.write("\n")
 	exit(error_number)
 
-def run_all_checks(ignore_repo=False):
+def run_all_checks(ignore_repo: bool = False) -> None:
 	assert_python_new_enough()
 	if not ignore_repo:
 		assert_git_repo()
