@@ -7,6 +7,9 @@ import re
 import shlex
 import shutil
 import gettext
+import functools
+import collections.abc
+import typing
 _ = gettext.gettext
 
 from .api_subprocess import Runner, CommandError, TimeoutExpired
@@ -16,6 +19,10 @@ from . import utils
 from . import settings
 from .constants import *
 
+if typing.TYPE_CHECKING:
+	from .main import App
+
+
 ALIGN_CHAR = "^"
 
 #https://matthew-brett.github.io/curious-git/git_object_types.html
@@ -44,6 +51,46 @@ error_no_unreachable_commits = _('no unreachable commits')
 hint_timeout = _("Either speed up the command (e.g. in case of a grep ignoring untracked files can significantly reduce the amount of files to be searched - depending on the repository and the working directory) or change the timeout in the config file with `set log-view.timeout=<s>` or `set details-view.timeout=<s>` where <s> is the time in seconds or -1 to disable the timeout.")
 
 
+# ---------- decorators ----------
+
+class CommandFunction:
+
+	def __init__(self,
+		method: 'collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], list[tuple[str, str, str]]] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], list[str|list[object]|tuple[str, object]]]',
+		arguments: str,
+		description: str,
+		settings: 'collections.abc.Sequence[str]',
+	) -> None:
+		self.method = method
+		self.arguments = arguments
+		self.description = description
+		self.settings = settings
+		functools.update_wrapper(self, method)
+
+	@typing.overload
+	def __call__(self, model: 'LogModel', args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
+		pass
+
+	@typing.overload
+	def __call__(self, model: 'DetailsModel', args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
+		pass
+
+	def __call__(self, model: 'typing.Any', args: 'collections.abc.Sequence[str]') -> 'typing.Any':
+		return self.method(model, args)
+
+
+def command_function(
+	arguments: str,
+	description: str,
+	settings: 'collections.abc.Sequence[str]',
+) -> 'collections.abc.Callable[[collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], list[tuple[str, str, str]]] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], list[str|list[object]|tuple[str, object]]]], CommandFunction]':
+
+	def decorator(method: 'collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], list[tuple[str, str, str]]] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], list[str|list[object]|tuple[str, object]]]') -> CommandFunction:
+		return CommandFunction(method, arguments=arguments, description=description, settings=settings)
+
+	return decorator
+
+
 # ---------- model classes ----------
 
 class NameSpace:
@@ -94,13 +141,13 @@ class Model(Runner):
 
 	CARRIAGE_RETURN = '' #r'\r'
 
-	cmd = None
+	cmd: 'list[str]|list[list[str]]'
 
 	# commands starting with this prefix are not executed as shell commands
 	# but looked up in command_functions
 	INTERNAL_FUNCTION_PREFIX = "$"
 
-	command_functions = dict()
+	command_functions: 'collections.abc.Mapping[str, CommandFunction]' = {}
 
 	ignore_returncode = False
 
@@ -278,6 +325,8 @@ _cmd_grep_todo = ["git", "grep", "--no-index", "--exclude-standard", "-n", "-I",
 
 class LogModel(Model):
 
+	app: 'App'   # is set in App.__init__
+
 	fmt_log    = "%Cred%h%Creset -^%C(yellow)%d%Creset %s %Cgreen(%cd) %C(bold blue)<%an>%Creset"
 	fmt_reflog = "%Cred%h %Creset%C(yellow)%gD%Creset^%C(yellow)%d%Creset %gs %Cgreen(%cd) %C(bold blue)<%gn>%Creset"
 
@@ -354,7 +403,7 @@ class LogModel(Model):
 
 	WC_LOGENTRY = '{logentry}'
 	ln_pattern = '* ^<color=default>' + WC_LOGENTRY
-	ln_pattern_error = '\n<color=%color.echo.error%>{err}</color>\nsee %{setting}%\ncmd = {cmd}\n{hint}'
+	ln_pattern_error = '^\n<color=%color.echo.error%>{err}</color>\nsee %{setting}%\ncmd = {cmd}\n{hint}'
 	ln_pattern_error_hint = '\n{hint}\n\n'
 
 	show_untracked = True
@@ -395,10 +444,26 @@ class LogModel(Model):
 		ln = utils.colored_str_to_markup(ln, self.app.define_color)
 		return [(ln, None, TYPE_ERROR)]
 
-	def status(self, args):
+	@command_function(
+		arguments = '',
+		description = _('Show untracked, unstaged and staged changes if existing'),
+		settings = [
+			'log-view.show-untracked',
+			'log-view.show-untracked-as-separate-group',
+			'log-view.line-pattern',
+			'title.untracked',
+			'title.unstaged',
+			'title.untracked-and-unstaged',
+			'title.staged',
+			settings.COLOR_LOG_UNTRACKED,
+			settings.COLOR_LOG_UNSTAGED,
+			settings.COLOR_LOG_STAGED,
+		],
+	)
+	def status(self, args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
 		# uncommitted changes
 		if args:
-			cmd = ['$status'] + args
+			cmd = ['$status'] + list(args)
 			raise CommandError(cmd, "status takes no arguments")
 
 		self.cmd = ["git", "status", "--porcelain=v1"]
@@ -421,7 +486,7 @@ class LogModel(Model):
 		if not self.show_untracked:
 			untracked = False
 
-		log = []
+		log: 'list[tuple[str, str, str]]' = []
 		if staged:
 			ln = self.format_log_entry(settings.COLOR_LOG_STAGED, title_staged)
 			log.insert(0, (ln, ID_STAGED, TYPE_OTHER))
@@ -441,27 +506,26 @@ class LogModel(Model):
 
 		return log
 
-	status.arguments = ''
-	status.description = _('Show untracked, unstaged and staged changes if existing')
-	status.settings = [
-		'log-view.show-untracked',
-		'log-view.show-untracked-as-separate-group',
-		'log-view.line-pattern',
-		'title.untracked',
-		'title.unstaged',
-		'title.untracked-and-unstaged',
-		'title.staged',
-		settings.COLOR_LOG_UNTRACKED,
-		settings.COLOR_LOG_UNSTAGED,
-		settings.COLOR_LOG_STAGED,
-	]
 
-	def stashes(self, args):
+	@command_function(
+		arguments = '',
+		description = _('Show stashed changes if existing'),
+		settings = [
+			'log-view.cmd.stash-list',
+			'log-view.show-stashes',
+			'log-view.show-stashes-as-group',
+			'log-view.show-stashes-in-reverse-order',
+			'log-view.line-pattern',
+			'title.stashes',
+			settings.COLOR_LOG_STASHES,
+		],
+	)
+	def stashes(self, args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
 		if args:
-			cmd = ['$stashes'] + args
+			cmd = ['$stashes'] + list(args)
 			raise CommandError(cmd, "stashes takes no arguments")
 
-		log = []
+		log: 'list[tuple[str, str, str]]' = []
 
 		if self.show_stashed_changes:
 			self.cmd = self.cmd_stash_list
@@ -481,21 +545,22 @@ class LogModel(Model):
 
 		return log
 
-	stashes.arguments = ''
-	stashes.description = _('Show stashed changes if existing')
-	stashes.settings = [
-		'log-view.cmd.stash-list',
-		'log-view.show-stashes',
-		'log-view.show-stashes-as-group',
-		'log-view.show-stashes-in-reverse-order',
-		'log-view.line-pattern',
-		'title.stashes',
-		settings.COLOR_LOG_STASHES,
-	]
 
-	def todo(self, args):
+	@command_function(
+		arguments = '',
+		description = _('Show TODO flags'),
+		settings = [
+			'log-view.cmd.todo.grep',
+			'log-view.show-todo-flags',
+			'log-view.show-todo-flags-on-timeout',
+			'log-view.line-pattern',
+			'title.todo.log-view',
+			settings.COLOR_LOG_TODO,
+		],
+	)
+	def todo(self, args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
 		if args:
-			cmd = ['$todo'] + args
+			cmd = ['$todo'] + list(args)
 			raise CommandError(cmd, "todo takes no arguments")
 
 		out_true = [(self.format_log_entry(settings.COLOR_LOG_TODO, title_todo_log), ID_TODO, TYPE_TODO)]
@@ -522,16 +587,6 @@ class LogModel(Model):
 		else:
 			return out_true
 
-	todo.arguments = ''
-	todo.description = _('Show TODO flags')
-	todo.settings = [
-		'log-view.cmd.todo.grep',
-		'log-view.show-todo-flags',
-		'log-view.show-todo-flags-on-timeout',
-		'log-view.line-pattern',
-		'title.todo.log-view',
-		settings.COLOR_LOG_TODO,
-	]
 
 	command_functions = dict(
 		status = status,
@@ -617,6 +672,8 @@ class UnreachableModel(LogModel):
 
 class DetailsModel(Model):
 
+	app: 'App'   # is set in App.__init__
+
 	WC_HASH_ID = settings.WC_HASH_ID
 
 	cmd_pattern = [
@@ -1101,7 +1158,7 @@ class DetailsModel(Model):
 		if self.reo_escape.search(fn):
 			try:
 				fn = self.reo_escape.sub(replace, fn)
-			except unicodedecodeerror:
+			except UnicodeDecodeError:
 				pass
 
 		return fn
@@ -1112,9 +1169,18 @@ class DetailsModel(Model):
 	todo_strip_indentation = True
 	todo_only = False
 
-	def todo(self, args):
+	@command_function(
+		arguments = '',
+		description = _('Show a list of the TODO flags in the code'),
+		settings = [
+			'details-view.cmd.todo.grep',
+			'details-view.todo.line-pattern',
+			'title.todo.details-view',
+		],
+	)
+	def todo(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
 		if args:
-			cmd = ['$todo'] + args
+			cmd = ['$todo'] + list(args)
 			raise CommandError(cmd, "todo takes no arguments")
 
 		timeout_expired = None
@@ -1176,15 +1242,13 @@ class DetailsModel(Model):
 
 		return out
 
-	todo.arguments = ''
-	todo.description = _('Show a list of the TODO flags in the code')
-	todo.settings = [
-		'details-view.cmd.todo.grep',
-		'details-view.todo.line-pattern',
-		'title.todo.details-view',
-	]
 
-	def referencedby(self, args):
+	@command_function(
+		arguments = 'CURRENT_COMMIT_HASH',
+		description = _('Show a list of commits referencing the current commit'),
+		settings = ['details-view.cmd.commit.referenced-by', 'details-view.commit.line-pattern-referenced-by'],
+	)
+	def referencedby(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
 		assert len(args) == 1
 		hash_id = args[0]
 
@@ -1208,11 +1272,13 @@ class DetailsModel(Model):
 
 		return out
 
-	referencedby.arguments = 'CURRENT_COMMIT_HASH'
-	referencedby.description = _('Show a list of commits referencing the current commit')
-	referencedby.settings = ['details-view.cmd.commit.referenced-by', 'details-view.commit.line-pattern-referenced-by']
 
-	def untracked(self, args):
+	@command_function(
+		arguments = '',
+		description = _('Return untracked files'),
+		settings = ['details-view.show-untracked-relative-to'],
+	)
+	def untracked(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
 		assert len(args) == 0
 
 		self.cmd = ["git", "status", "--porcelain=v1"]
@@ -1234,20 +1300,34 @@ class DetailsModel(Model):
 
 		return files
 
-	untracked.arguments = ''
-	untracked.description = _('Return untracked files')
-	untracked.settings = ['details-view.show-untracked-relative-to']
 
 	indent_body = _("    ")
-	def commitheader(self, args):
+
+	@command_function(
+		arguments = 'CURRENT_COMMIT_HASH',
+		description = _('Show the meta info of the current commit (hash, branch, tag, author, committer, date, message)'),
+		settings = [
+			'details-view.cmd.commit.header-data',
+			'details-view.commit.user-pattern',
+			'title.commit.refnames.head-branch-sep',
+			settings.COLOR_TITLE,
+			settings.COLOR_DECORATION_HEAD,
+			settings.COLOR_DECORATION_HEAD_BRANCH_SEP,
+			settings.COLOR_DECORATION_BRANCH_LOCAL,
+			settings.COLOR_DECORATION_BRANCH_REMOTE,
+			settings.COLOR_DECORATION_TAG,
+		],
+	)
+	def commitheader(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
 		assert len(args) == 1
 		hash_id = args[0]
 
-		out = []
+		out: 'list[str|list[object]|tuple[str, object]]' = []
 		values = self.get_commit_header_data(hash_id)
 		if values is None:
 			return []
 		refnames = values.pop('refnames')
+		markup_refnames: 'str|list[object]|tuple[str, object]'
 		if refnames:
 			markup_refnames = utils.replace_to_markup(refnames, ', ', refnames_sep, markup_function=self.markup_deco)
 			markup_refnames = utils.replace_to_markup(pattern_refnames, '{refnames}', markup_refnames)
@@ -1359,20 +1439,6 @@ class DetailsModel(Model):
 
 		return decoration
 
-	commitheader.arguments = 'CURRENT_COMMIT_HASH'
-	commitheader.description = _('Show the meta info of the current commit (hash, branch, tag, author, committer, date, message)')
-	commitheader.settings = [
-		'details-view.cmd.commit.header-data',
-		'details-view.commit.user-pattern',
-		'title.commit.refnames.head-branch-sep',
-		settings.COLOR_TITLE,
-		settings.COLOR_DECORATION_HEAD,
-		settings.COLOR_DECORATION_HEAD_BRANCH_SEP,
-		settings.COLOR_DECORATION_BRANCH_LOCAL,
-		settings.COLOR_DECORATION_BRANCH_REMOTE,
-		settings.COLOR_DECORATION_TAG,
-	]
-
 
 	command_functions = dict(
 		referencedby = referencedby,
@@ -1530,6 +1596,17 @@ def get_relative_path(path):
 	cwd = os.path.relpath(cwd, root) + os.sep
 	return os.path.relpath(path, cwd)
 
+def get_current_branch() -> 'str|None':
+	'''
+	returns None in detached HEAD state
+	'''
+	cmd = ['git', 'symbolic-ref', '--short', 'HEAD']
+
+	try:
+		return Runner().run_and_get_output(cmd).rstrip()
+	except CommandError:
+		return None
+
 
 # ---------- opener ----------
 
